import { routerRedux } from 'dva/router';
import { login, userInfo } from '../services/login';
import { setCookie, fullPath } from '../utils/helper';
import { cookie } from '../utils/config';
import { errors } from '../utils/error';

export default {
  namespace: 'login',
  state: {},
  reducers: {},
  effects: {
    *login({ payload }, { call, put }) {
      const result = yield call(login, payload);
      const { tokenId, authResponse, remainedAuthRequirements } = result;
      if (authResponse.status !== 'authed' && authResponse.status !== 'skipped') {
        throw errors.wrongPassword();
      }
      const { requirements } = remainedAuthRequirements;
      if (requirements.length > 0) {
        throw errors.unsupportedAuthType(requirements);
      }
      setCookie(cookie.token, tokenId);
      const uInfo = yield call(userInfo);
      setCookie(cookie.userId, uInfo.id);
      setCookie(cookie.userName, uInfo.name);
      yield put(routerRedux.push(fullPath('/domain')));
    },
  },
  subscriptions: {},
};
